/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#pragma ident	"@(#)misc.c	1.3	99/02/18 SMI"

#include	<errno.h>
#include	<sys/types.h>
#ifndef SOLARIS
#include	<sys/v86.h>
#endif
#include	<sys/sysi86.h>
#include	"lxrun.h"

int sysi86(int cmd, ...);

/*
 * assorted random system calls
 */

/*
 *	ioperm		(PARTLY IMPLEMENTED)
 *	iopl		(OSR5 only)
 *	vhangup		(NOT IMPLEMENTED)
 *	idle		(NOT IMPLEMENTED)
 *	vm86		(NOT IMPLEMENTED)
 *	modify_ldt	(NOT IMPLEMENTED)
 *	clone		(NOT IMPLEMENTED)
 *	sysctl		(NOT IMPLEMENTED)
 */

int
lx_ioperm(unsigned long fron, unsigned long num, int turn_on)
{
#if defined(OSR5) || defined(SOLARIS) 
	return sysi86(SI86V86, V86SC_IOPL, turn_on ? 0x3000 : 0);
#elif defined(UNIXWARE) || defined(GEMINI)
	/* Untested.  Is this correct?  */
	return sysi86(SI86IOPL, turn_on ? 3 : 0);
#else
	return enosys("ioperm");
#endif
}

int
lx_iopl(int level)
{
#if defined(OSR5)
	if (level > 3 || level < 0) {
		errno = EINVAL;
		return -1;
	}

	return sysi86(SI86V86, V86SC_IOPL, level << 24);
#elif defined(SOLARIS)
	/* XXX: untested */
	if (level > 3 || level < 0) {
		errno = EINVAL;
		return -1;
	}

	return sysi86(SI86V86, V86SC_IOPL, level << 12);
#else
	return enosys("iopl");
#endif
}

int lx_vhangup()	{ return enosys("vhangup"); }
int lx_idle()		{ return enosys("idle"); }
int lx_vm86()		{ return enosys("vm86"); }
int lx_modify_ldt()	{ return enosys("modify_ldt"); }
int lx_clone()		{ return enosys("clone"); }
int lx_sysctl()		{ return enosys("sysctl"); }

/*
 * setup() is only called at boot, early on in /sbin/init.
 * acct() is for process accounting.
 * ustat() is some wacky pre-statfs() statfs().
 * 
 * The rest aren't even implemented in Linux.
 */
int lx_acct()		{ return enosys("acct"); }
int lx_afs_syscall()	{ return enosys("afs_syscall"); }
int lx_break()		{ return enosys("break"); }
int lx_ftime()		{ return enosys("ftime"); }
int lx_gtty()		{ return enosys("gtty"); }
int lx_lock()		{ return enosys("lock"); }
int lx_mpx()		{ return enosys("mpx"); }
int lx_phys()		{ return enosys("phys"); }
int lx_prof()		{ return enosys("prof"); }
int lx_profil()		{ return enosys("profil"); }
int lx_setup()		{ return enosys("setup"); }
int lx_stty()		{ return enosys("stty"); }
int lx_ulimit()		{ return enosys("ulimit"); }
int lx_ustat()		{ return enosys("ustat"); }
