/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#pragma ident	"@(#)sem.c	1.4	99/02/18 SMI"

#include	<errno.h>
#include	<string.h>
#include	<sys/sem.h>
#include	"lxrun.h"
#include	"linux/sem.h"

union semun {
	int             val;
	struct semid_ds *buf;
	ushort_t        *array;
} arg;

static int
copy_in_semid_ds(struct lx_semid_ds *from, struct semid_ds *to)
{
	if (! from || !verify_read(from, sizeof(*from)))
		return -1;

	memset(to, 0, sizeof(*to));

	to->sem_perm.key	= from->sem_perm.key;
	to->sem_perm.uid	= from->sem_perm.uid;
	to->sem_perm.gid	= from->sem_perm.gid;
	to->sem_perm.cuid	= from->sem_perm.cuid;
	to->sem_perm.cgid	= from->sem_perm.cgid;
	to->sem_perm.mode	= from->sem_perm.mode;
	to->sem_perm.seq	= from->sem_perm.seq;

	to->sem_otime		= from->sem_otime;
	to->sem_ctime		= from->sem_ctime;
	to->sem_nsems		= from->sem_nsems;

	return 0;
}


static int
copy_out_semid_ds(struct semid_ds *from, struct lx_semid_ds *to)
{
	if (! to || !verify_write(from, sizeof(*to)))
		return -1;

	memset(to, 0, sizeof(*to));

	to->sem_perm.key	= from->sem_perm.key;
	to->sem_perm.uid	= from->sem_perm.uid;
	to->sem_perm.gid	= from->sem_perm.gid;
	to->sem_perm.cuid	= from->sem_perm.cuid;
	to->sem_perm.cgid	= from->sem_perm.cgid;
	to->sem_perm.mode	= from->sem_perm.mode;
	to->sem_perm.seq	= from->sem_perm.seq;

	to->sem_otime		= from->sem_otime;
	to->sem_ctime		= from->sem_ctime;
	to->sem_nsems		= from->sem_nsems;

	return 0;
}

int
lx_semget(key_t key, int nsems, int lx_flag)
{
	int flag = lx_flag & 0777;

	if (lx_flag & LX_IPC_CREAT)
		flag	|= IPC_CREAT;
	if (lx_flag & LX_IPC_EXCL)
		flag	|= IPC_EXCL;
	return (semget(key, nsems, flag));
}

int
lx_semop(int semid, struct lx_sembuf *sops, unsigned nsops)
{
	return (semop(semid, (struct sembuf *)sops, nsops));
}

int
lx_semctl(int semid, int semnum, int cmd, union semun arg)
{
	int r = -1;
	struct semid_ds sem_data;
	union semun arg2;

	switch (cmd) {
	case LX_IPC_SET:
		if ((r = copy_in_semid_ds((void *)arg.buf, &sem_data)) == 0) {
			arg2.buf = &sem_data;
			r = semctl(semid, semnum, IPC_SET, arg2);
		}
		break;
	case LX_IPC_STAT:
		if ((r = semctl(semid, semnum, IPC_STAT, &sem_data)) == 0) {
			r = copy_out_semid_ds(&sem_data, (void *)arg2.buf);
		}
		break;
	case LX_IPC_RMID:
		r = semctl(semid, semnum, IPC_RMID, arg);
		break;
	case LX_GETALL:
		r = semctl(semid, semnum, GETALL, arg);
		break;
	case LX_GETNCNT:
		r = semctl(semid, semnum, GETNCNT, arg);
		break;
	case LX_GETPID:
		r = semctl(semid, semnum, GETPID, arg);
		break;
	case LX_GETVAL:
		r = semctl(semid, semnum, GETVAL, arg);
		break;
	case LX_GETZCNT:
		r = semctl(semid, semnum, GETZCNT, arg);
		break;
	case LX_SETALL:
		r = semctl(semid, semnum, SETALL, arg);
		break;
	case LX_SETVAL:
		r = semctl(semid, semnum, SETVAL, arg);
		break;
	default:
		r = enosys("semctl subfunction");
		break;
	}
	return (r);
}
